# Copyright 2013 David Reichard
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
###############################################################################
import pygame
import gamelib.event


class SceneGroup(pygame.sprite.LayeredUpdates):
    def __init__(self, parent, *sprites, **kwargs):
        self.parent = parent
        pygame.sprite.LayeredUpdates.__init__(self, *sprites, **kwargs)

    def add_internal(self, scene_object, layer=None):
        scene_object.on_add(self)
        scene_object.parent = self
        pygame.sprite.LayeredUpdates.add_internal(self, scene_object, layer)

    def remove_internal(self, scene_object):
        """ Make sure to deafen any events the object is listening to. """
        scene_object.parent = None

        for key in self.parent.event_types.keys():
            for func in self.parent.event_types[key]:
                if hasattr(func, 'im_self') and func.im_self is scene_object:
                    self.parent.deafen(key, func)

        pygame.sprite.LayeredUpdates.remove_internal(self, scene_object)


class Scene(gamelib.event.EventDispatcher):
    def __init__(self, *args, **kwargs):
        gamelib.event.EventDispatcher.__init__(self)
        self.groups = {}
        self.group_list = []
        self.on_create(*args, **kwargs)

    def on_create(self, *args, **kwargs):
        # TODO: Call attached script to initialize any event interests with self, set
        # TODO: attributes, etc...
        pass

    def add(self, group_name, *sprites, **kwargs):
        if not group_name in self.groups:
            new_group = SceneGroup(self)
            self.groups[group_name] = new_group
            self.group_list.append(new_group)

        self.groups[group_name].add(*sprites, **kwargs)

    def remove(self, group_name, *sprites):
        if not group_name in self.groups:
            raise IndexError('Unable to remove sprite(s), group ' + group_name + ' does not exist.')

        self.groups[group_name].remove(*sprites)

    def draw(self, surface):
        """ Draw the entire scene.

        Keyword arguments:
        surface = The display surface to draw on.

        """
        for group in self.group_list:
            pygame.sprite.LayeredUpdates.draw(group, surface)

    def update(self, interval):
        for group in self.group_list:
            group.update(interval)