# Copyright 2013 David Reichard
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
###############################################################################


class Timer(object):
    """
        Decorator for functions called in update methods that need a timed call.
        Example:

        @Timer(5)
        def called_every_5_seconds:
            ...do something
    """
    def __init__(self, interval):
        self.counter = 0.0
        self.interval = interval

    def __call__(self, f):
        def wrapped_f(*args):
            self.counter += args[1]
            if self.counter >= self.interval:
                self.counter -= self.interval
                arg_list = [args[0]] + list(args[2:])
                f(*arg_list)
        return wrapped_f